/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.abstractfactory;

import dk.rode.thesis.bridge.MemorizableSequenceAbstraction;
import dk.rode.thesis.meta.Participant;
import dk.rode.thesis.meta.ParticipantUsage;
import dk.rode.thesis.meta.log.LogFactory;
import dk.rode.thesis.strategy.ObjectPolicy;

/**
  * A <i>memorizable abstraction factory</i> creates 
  * {@link MemorizableSequenceAbstraction memorizable} sequence
  * abstractions. <p>
  * 
  * This factory is thread-safe. <p>
  *
  * <b>Implementation notes</b>:<br>
  * The factory method use a covariant return type since the
  * {@code MemorizableSequenceAbstraction} class offers new
  * functionality compared to the {@code SequenceAbstraction} class.
  * 
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  * 
  * @see    StandardAbstractionFactory
  * @see    SynchronisedAbstractionFactory
  * @see    PrototypicalAbstractionFactory
  * 
  * @param  <E> The type of values delivered by the created abstractions.
  */
@ParticipantUsage(value = "ConcreteProduct", type = MemorizableSequenceAbstraction.class)
@Participant("ConcreteFactory")
public class MemorizableAbstractionFactory<E> implements AbstractionFactory<E>  {
    
    /**
      * Constructor.
      */
    public MemorizableAbstractionFactory() {
        super();
    }
    
    /**
      * 
      */
    public MemorizableSequenceAbstraction<E> createAbstraction() {
        MemorizableSequenceAbstraction<E> abstraction = new MemorizableSequenceAbstraction<E>();
        LogFactory.getLog(this).println("Created memorizable abstraction: ", abstraction);
        return abstraction;
    }

    /**
      * Returns the string representation of this factory.
      * 
      * @return The string representation; never null.
      */
    public String toString() {
        return ObjectPolicy.IDENTITY.asString(this);
    }
    
}
