/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.chainofresponsibility;

/**
  * A <i>handler link</i> represents a connection to the next
  * {@link Handler handler} in a given <i>handler chain</i>. <p> 
  * 
  * A link is invoked from the previous handler that could not
  * handle a request, forwarding the both the request and
  * the (unsuccessful) handler itself. <p>
  * 
  * A handler chain will normally be represented as a 
  * {@link HandlerChain} object, but this is not a requirement.
  *
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  * 
  * @param  <R> The request type handled by connected handlers.
  */
public interface HandlerLink<R> {

    /**
      * Forwards the the request supplied as {@code request} 
      * to the {@link Handler handler} represented by this link. <p>
      * 
      * The next handler is the handler after {@code current} in
      * the chain. {@code current} can be located at any place
      * in the chain, and if not found, all handlers in the chain
      * are tried.
      * 
      * @param  request The request to handle; nullability determined
      *                 by the actual handler implementation.
      * @param  current The handler in the chain that could <b>not</b>
      *                 handle {@code request}; can be null if this
      *                 link represents the first handler in the chain.
      * @return The handler that handled {@code request}, if any. 
      *         Can be null.
      */
    public Handler<R> forward(R request, Handler<R> current);
    
}
