/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 * 
 */

/**
  * Implementations and examples of the <b>Chain of Responsibility</b> 
  * design pattern [<a href="http://www.rode.dk/thesis/Bibliography.htm#Gamma95" target="thesis-bibliography">Gamma95</a>, p.223]. <p>
  *  
  * <b>Intent</b>:<ul><p><i>
  * 
  *    Avoid coupling the sender of a request to its receiver by giving 
  *    more than one object a chance to handle the request. Chain the 
  *    receiving objects and pass the request along the chain until an 
  *    object handles it.
  *    
  * </i></ul>
  * 
  * Here, the <b>Handler</b> participant is represented by the
  * {@link dk.rode.thesis.chainofresponsibility.Handler}
  * interface, and the <b>ConcreteHandler</b> participant by any 
  * {@code Handler} implementation. Several handler implementations
  * are defined in this package, all sub-classes of the 
  * {@link dk.rode.thesis.chainofresponsibility.CharacterHandler}  
  * class, each thus handling {@link dk.rode.thesis.flyweight.Character}
  * objects. Examples include {@link dk.rode.thesis.chainofresponsibility.SymbolHandler}
  * and {@link dk.rode.thesis.chainofresponsibility.LetterCaseHandler}. <p>
  * 
  * The <b>Client</b> participant is the test application, i.e. the 
  * {@link dk.rode.thesis.chainofresponsibility.Main} class. <p>
  * 
  * <a name="uml"></a><b>UML Class Diagram</b>:<br>
  * <img src="doc-files/ChainOfResponsibility.png" border="0"><p>
  * 
  * <b>Implementation notes</b>:<br/> 
  * This implementation differs from the canonical "Gang of Four" 
  * implementation in that it makes the <i>handler chain</i> explicit using
  * the {@link dk.rode.thesis.chainofresponsibility.HandlerChain} 
  * interface. Here, handlers need not store the successor in the chain locally, 
  * and can participate in several chains concurrently. The link to the
  * next handler is supplied at execution time using a 
  * {@link dk.rode.thesis.chainofresponsibility.HandlerLink}. <p>
  * 
  * Two explicit {@code HandlerChain} implementations have been made,
  * namely {@link dk.rode.thesis.chainofresponsibility.StandardHandlerChain}
  * and {@link dk.rode.thesis.chainofresponsibility.WeakHandlerChain}. The first
  * is a simple implementation that is not thread-safe, while the latter is
  * thread-safe and stores handlers as weak references.     
  *  
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a> 
  */
@Pattern(name         = "Chain of Responsibility", 
         scope        = Scope.Object,
         purpose      = Purpose.Behavioural,
         participants = {"Handler", "ConcreteHandler", "Client"})
package dk.rode.thesis.chainofresponsibility;

import dk.rode.thesis.meta.Pattern;
import dk.rode.thesis.meta.Purpose;
import dk.rode.thesis.meta.Scope;






