/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.flyweight;

import dk.rode.thesis.meta.Participant;

/**
  * A <i>whitespace</i> represents a single white
  * space character.
  *
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  */
@Participant("ConcreteFlyweight")
public class Whitespace extends AbstractCharacter {

    /**
      * White space representation. <p>
      * 
      * No internal state as all white space is represented as
      * a single space... Tabs and newlines considered harmful... :)
      */
    final static String space = " ";

    /**
      * No-arg constructor.
      */
    Whitespace() {
        super();
    }

    /**
      * 
      */
    public final char charAt(int index) {
        return space.charAt(index);
    }

    /**
      * 
      */
    public final int length() {
        return 1;
    }

    /**
      * 
      */
    public final CharSequence subSequence(int start, int end) {
        return space.subSequence(start, end);
    }

    /**
      * 
      */
    public final String toString() {
        return space;
    }
    
    /**
      * 
      */
    public final int compareTo(Character character) {
        return space.compareTo(character.toString());
    }

}
