/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */

/**
  * Implementations and examples of the <b>Flyweight</b> 
  * design pattern [<a href="http://www.rode.dk/thesis/Bibliography.htm#Gamma95" target="thesis-bibliography">Gamma95</a>, p.195]. <p>
  *  
  * <b>Intent</b>:<ul><p><i>
  *    
  *    Use sharing to support large numbers of finegrained 
  *    objects efficiently.
  *    
  * </i></ul>
  * 
  * Here, the <b>Flyweight</b> participant is represented by the 
  * {@link dk.rode.thesis.flyweight.Textual} interface. 
  * This package supplies three flyweight types, namely the 
  * {@link dk.rode.thesis.flyweight.Character} interface and 
  * the {@link dk.rode.thesis.flyweight.Word} and 
  * {@link dk.rode.thesis.flyweight.Sentence} classes. <p>
  * 
  * A standard abstract implementation of the {@code Character} interface
  * is supplied in form of the {@link dk.rode.thesis.flyweight.AbstractCharacter} 
  * class, and each implementation corresponds to the <b>ConcreteFlyweight</b>
  * participant, for example {@link dk.rode.thesis.flyweight.Letter}
  * and {@link dk.rode.thesis.flyweight.Symbol}. <p>
  * 
  * The {@code Word} class also represent the <b>ConcreteFlyweight</b> 
  * participant, while the {@code Sentence} class represents the
  * <b>UnsharedConcreteFlyweight</b> participant. <p>
  * 
  * Flyweights are constructed using the <b>FlyweightFacctory</b> 
  * participant, which is represented by the
  * {@link dk.rode.thesis.flyweight.CharacterFactory} class. <p>  
  *
  * The <b>Client</b> participant is the test application, i.e. the 
  * {@link dk.rode.thesis.flyweight.Main} class. <p>
  * 
  * <a name="uml"></a><b>UML Class Diagram</b>:<br>
  * <img src="doc-files/Flyweight.png" border="0"><p>
  * 
  * <b>Implementation notes</b>:<br/>
  * Both {@code Word} and {@code Sentence} are in fact implementations
  * of the {@link dk.rode.thesis.meta.model.ArraySequence} class: 
  * the <i>elements</i> of a sentence is the words it contains, while the 
  * elements of a word is the letters it contains. Hence, {@code Sentence},
  * {@code Word}, and {@code Character} objects form a 
  * {@linkplain dk.rode.thesis.composite Composite}-like structure. This fits
  * very well with the general semantics of {@code Sequence}, for example
  * that any {@code Word} must have at least one {@code Character},
  * or a {@code Sequence} at least one {@code Word}. <p>
  * 
  * All textual flyweights in this package are also 
  * {@link dk.rode.thesis.strategy.Stringable}. <p>
  * 
  * The internal states used by the {@link dk.rode.thesis.state.ReversiblePrimeSequence}
  * class can be seen as a degenerate application of the Flyweight pattern. 
  * 
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a> 
  */
@Pattern(name         = "Flyweight", 
         scope        = Scope.Object,
         purpose      = Purpose.Structural,
         participants = {"Flyweight", 
                         "ConcreteFlyweight", 
                         "UnsharedConcreteFlyweight",
                         "FlyweightFactory",
                         "Client"})
package dk.rode.thesis.flyweight;

import dk.rode.thesis.meta.Pattern;
import dk.rode.thesis.meta.Purpose;
import dk.rode.thesis.meta.Scope;






