/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.interpreter;

import dk.rode.thesis.strategy.ObjectPolicy;

/**
  * An <i>abstract expression</i> represents the basic
  * traits of any {@link Expression expression}. <p>
  * 
  * An abstract expression is stateless. 
  * 
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  * 
  * @param  <E> The type of value the evaluation of this expression 
  *             produces.             
  */
public abstract class AbstractExpression<E> implements Expression<E> {
    
    /**
      * No-arg sub-class constructor.
      */
    protected AbstractExpression() {
        super();
    }

    /**
      * 
      */
    public boolean contains(Expression<?> expression) {
        if (this == expression) {
            return true;
        }
        for (Expression<?> e : this.operands()) {
            if (e.contains(expression)) {
                return true;
            }
        }
        return false;
    }

    /**
      * Returns the hash code of this expression.
      * 
      * @return The hash code; never null.
      */
    public int hashCode() {
        return 17 * this.name().hashCode() + 31 * this.operands().hashCode();
    }
    
    /**
      * Returns true if {@code object} is an instance of this
      * expression type having the same {@link Expression#name() name}
      * and all {@link Expression#operands() operands} are 
      * equal using {@code equals(Object)}, respectively,
      * false otherwise.
      * 
      * @param  object The object to test; can be null.
      * @return True if equal, false if not.
      */
    public boolean equals(Object object) {
       if (object == null) {
           return false;
       } else if (object == this) {
           return true;
       } else if (this.getClass().equals(object.getClass())) {
           Expression<?> e = (Expression<?>)object; 
           if (this.name().equals(e.name())) {
               return this.operands().equals(e.operands());
           }
       }
       return false;
    }
    
    /**
      * Returns the string representation of this expression.
      * 
      * @return The string representation; never null.
      */
    public String toString() {
        StringBuilder sb = (StringBuilder)ObjectPolicy.IDENTITY_OPEN.toString(this);
        return sb.append("; operands: ").append(this.operands()).append(']').toString();
    }
    
}
