/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.interpreter;

/**
  * A <i>typed expression</i> exposes the type of the value its 
  * evaluation produces via its {@link #type()} method. <p> 
  *
  * Only typed expressions can be the target of a {@link BreakExpression}. <p>
  *  
  * Break expressions are the <b>only</b> type of expression that is 
  * allowed to return null by the {@code type()} method.
  * 
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  * 
  * @see    TypedExpressionDecorator
  * 
  * @param  <E> The type of value the evaluation of this expression 
  *             produces.
  */
public interface TypedExpression<E> extends Expression<E> {

    /**
      * Return the type of value the evaluation of this expression 
      * produces.
      * 
      * @return The type. The <b>only</b> expression type allowed to
      *         return null is {@link BreakExpression}!
      */
    public Class<E> type();
    
    /**
      * Copies this typed expression. <p>
      * 
      * Unlike {@link #asSymbol(Context)}, copying 
      * <b>cannot</b> handle cyclic expression references!
      * 
      * @return A copy of this typed expression; never null.
      */
    public TypedExpression<E> copy();
    
}
