/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 * 
 */

/**
  * Implementations and examples of the <b>Mediator</b> 
  * design pattern [<a href="http://www.rode.dk/thesis/Bibliography.htm#Gamma95" target="thesis-bibliography">Gamma95</a>, p.273]. <p>
  *  
  * <b>Intent</b>:<ul><p><i>
  * 
  *    Define an object that encapsulates how a set of objects interact. 
  *    Mediator promotes loose coupling by keeping objects from referring 
  *    to each other explicitly, and it lets you vary their interaction 
  *    independently 
  *         
  * </i></ul>
  *
  * Unfortunately, due to a lack of time, the Mediator pattern has not been
  * implemented. The idea for the pattern implementation was to construct a small 
  * "testframework", where Mediator would play a central role controlling the 
  * individual tests and the objects tested as well as the communication between 
  * the two. Akin to the ModelViewController pattern 
  * [<a href="http://www.rode.dk/thesis/Bibliography.htm#Buschmann96" target="thesis-bibliography">Buschmann96</a>, p.125], 
  * the Mediator would play the Controller part where sequences could be tested 
  * (Model) and the results presented (View). Model and View thus corresponds to 
  * what Gamma et al. describe as Colleague Classes.
  *  
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a> 
  */
@Pattern(name         = "Mediator", 
         scope        = Scope.Object,
         purpose      = Purpose.Behavioural,
         participants = {"Mediator", "ConcreteMediator", "Colleague"}) 
package dk.rode.thesis.mediator;

import dk.rode.thesis.meta.Pattern;
import dk.rode.thesis.meta.Purpose;
import dk.rode.thesis.meta.Scope;






