/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.memento;

import static dk.rode.thesis.meta.util.Throwables.toThrowableType;
import dk.rode.thesis.meta.Participant;
import dk.rode.thesis.meta.log.LogFactory;
import dk.rode.thesis.meta.model.EnglishAlphabetSequence;

/**
  * A <i>memorizable alphabet sequence</i> for letters in the 
  * English alphabet, i.e. {@code a} though {@code z}, lower-cased, 
  * that can save and restore its internal state in form of 
  * {@link SequenceMemento mementos}. <p>
  * 
  * <b>Implementation notes</b>:<br>
  * The mementos created are regular {@link SequenceMemento} instances, 
  * not {@link GuardedSequenceMemento} instances as in the 
  * {@link RangeSequence} class.
  *  
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  */
@Participant("Originator") 
public class MemorizableEnglishAlphabetSequence
     extends EnglishAlphabetSequence 
  implements MemorizableSequence<String> { 
    
    /**
      * No-arg constructor.
      */
    public MemorizableEnglishAlphabetSequence() {
        super();
    }

    /**
      * Copy constructor. <p>
      * 
      * This sequence starts at the current index of {@code sequence}.
      * 
      * @param  sequence The sequence to copy; cannot be null.
      * @throws NullPointerException If {@code sequence} is null.
      */
    public MemorizableEnglishAlphabetSequence(EnglishAlphabetSequence sequence) {
        super(sequence);
    }
    
    /**
      * 
      */
    public MemorizableEnglishAlphabetSequence copy() {
        return new MemorizableEnglishAlphabetSequence(this);
    }
    
    /**
      *
      */
    public SequenceMemento<String> getMemorizableState() {
        return new SequenceMemento<String>(this);
    }

    /**
      *
      */
    public void setMemorizableState(SequenceMemento<String> memento) throws MemorizableException {
        try {
            MemorizableEnglishAlphabetSequence meas = 
                (MemorizableEnglishAlphabetSequence)memento.getSequence();
            this.index = meas.index;
            this.state = meas.state;
            LogFactory.getLog(this).println("Updated ", this, " using ", memento);

        // To maintain memorizable sequence semantics...
        } catch (Exception e) {
            throw toThrowableType(e, MemorizableException.class);
        }
    }

}
