/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.meta.model;

/**
  * A <i>digit sequence</i> contains the 10 digits in
  * the decimal system, in order from {@code 0} to {@code 9}.
  *  
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  */
public final class DigitSequence extends ArraySequence<CharSequence> {

    /**
      * All digits as elements. 
      */
    private final static CharSequence[] digits = {
        "0", "1", "2", "3", "4", "5", "6", "7", "8", "9" 
    };

    /**
      * No-arg constructor.
      */
    public DigitSequence() {
        super(digits);
    }
    
    /**
      * Copy constructor. <p>
      * 
      * This sequence starts at the current index of {@code sequence}.
      * 
      * @param  sequence The sequence to copy; cannot be null.
      * @throws NullPointerException If {@code sequence} is null.
      */
    public DigitSequence(DigitSequence sequence) {
        super(sequence);
    }
    
    /**
      * 
      */
    public final DigitSequence copy() {
        return new DigitSequence(this);
    }
    
}
