/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.meta.reflect.proxy;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;

import dk.rode.thesis.meta.reflect.Reflection;
import dk.rode.thesis.strategy.ObjectPolicy;

/**
  * A <i>reference invocation handler</i> is an invocation handler that has
  * support for {@link Reference referenced} objects.
  * 
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  * 
  * @see    Reference
  * @see    ProxyFactory#getSharedObject(Object, String, Class[], String...)
  */
public class ReferenceInvocationHandler implements InvocationHandler {

    /**
      * The reference <i>currently</i> used by this handler. <p>
      * 
      * Never null.
      */
    private Reference<?> reference;

    /**
      * Constructor, which creates this handler.
      * 
      * @param reference The original reference to use; cannot be null.
      * @throws NullPointerException If {@code reference} is null.
      */
    public ReferenceInvocationHandler(Reference<?> reference) {
        if ((this.reference = reference) == null) {
            throw new NullPointerException("Reference cannot be null");
        }
    }

    /**
      * 
      */
    public Object invoke(Object proxy, Method method, Object[] arguments) throws Throwable {
        // Check to see if the method invoked is a specified
        // copy method to intercept...
        if (this.reference.copyMethod(method)) {
            return this.reference.addReference();
        }
        // Check to see if we must copy...
        this.reference = this.reference.copyIfMutator(method);
        return Reflection.invoke(this.reference.getReference(), method, arguments);
    }

    /**
      * Returns the string representation of this handler.
      * 
      * @return The string representation; never null.
      */
    public String toString() {
        StringBuilder sb = (StringBuilder)ObjectPolicy.IDENTITY_OPEN.toString(this);
        return sb.append("; reference: ").append(this.reference).append(']').toString();
    }

}
