/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.meta.util;

import dk.rode.thesis.meta.reflect.proxy.ProxyFactory;

/**
  * Various <i>constants</i> used in this project.
  *
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  */
public class Const {

    /**
      * Private off-limit constructor.
      */
    private Const() {
        throw new AssertionError(); // guard against reflection
    }
    
    /**
      * Constant representing the empty string.
      */
    public final static String EMPTY = "";
    
    /**
      * Common serial id used in all serializable classes. <p>
      * 
      * We rely on an explicit value rather than a computed one! <p>
      * 
      * The value is 1.
      */
    public final static long SERIAL_VERSION_UID = 1L;
    
    /**
      * Unique object identifying null where null cannot be used. <p>
      * 
      * The string representation of this null object is {@code NULL}.
      * 
      * @see ProxyFactory#getNullObject(Class)
      */
    public final static Object NULL = new Object() {
        public String toString() {
            return "NULL";
        }
    };

    /**
      * Value indicating that a given otherwise zero or positive number
      * is considered to be infinite. <p>
      * 
      * The value is minus one.
      */
    public final static transient int INFINITE = -1;

    /**
      * Value indicating <i>end-of-file</i>, <i>end-of-stream</i>,
      * <i>end-of-search</i>, etc. <p>
      * 
      * The value is minus one.
      */
    public final static transient int EOF = -1;
    
    /**
      * The Java name used for static initialisation blocks. <p>
      * 
      * The value is {@code <clinit>}.
      */
    
    public final static String STATIC_INITIALISER_NAME = "<clinit>";
    
    /**
      * The Java name used for constructors and instance 
      * initialisation blocks. <p>
      * 
      * The value is {@code <init>}.
      */
    public final static String CONSTRUCTOR_NAME = "<init>";
    
}
