/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.observer;

import dk.rode.thesis.meta.Participant;
import dk.rode.thesis.meta.model.Sequence;

/**
  * An <i>observable sequence</i> is a sequence that allows for
  * <i>observers</i> of the type supplied as the type parameter
  * {@code O} to be notified when the sequence changes its <i>aspect</i>.
  * The sequence aspect could for example the its internal state, and 
  * the type of aspect is determined by the type parameter {@code A}.
  * Observers subscribe to all possible values of the sequence
  * aspect or none. <p>
  * 
  * The notification of observers is performed by invoking the 
  * {@link SequenceObserver#sequenceEvent(Sequence, Object)}
  * method with this sequence as the first argument and the <i>aspect</i>
  * of this sequence as the second. This will normally be handled
  * by the implementing sequence itself, but how individual observers 
  * are notified must be implemented by the sub-class as it is  
  * dependent on the type of {@code O}. <p>
  *
  * The order in which observers are notified is undefined. <p>
  * 
  * <b>Implementation notes</b>:<br>
  * In case the type of observers is the {@link SequenceObserver} 
  * class ("the default"), the observable sequence in question is 
  * will double as an observer as well: observable sequence {@code X} 
  * can add observable sequence {@code Y} as an observer. The structure 
  * becomes {@linkplain dk.rode.thesis.composite composite} in a 
  * depth-first manner because the 
  * {@link SequenceObserver#sequenceEvent(Sequence, Object)}
  * method can then be used to <i>notify observers</i> and <i>to be 
  * notified from other observable sequences</i>. <p>
  * 
  * This interface declares no new methods besides inherited 
  * functionality.
  *
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  *
  * @see    AspectObservableSequence
  * 
  * @param  <O> The type of observers. 
  * @param  <A> The type of aspect used by this sequence to notify observers. 
  * @param  <E> The type of values delivered by this observable sequence. 
  */
@Participant({"Subject", "Observer"})
public interface ObservableSequence<O,A,E> 
         extends Observable<O>, 
                 Sequence<E>, 
                 SequenceObserver<A> {

    // All functionality supplied by the implemented 
    // interfaces and type parameters...
    
}
