/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.prototype;

import dk.rode.thesis.meta.Participant;

/**
  * A <i>strict copyable</i> object can return a <i>deep copy</i> of it self  
  * any type implementing this interface can act as a <i>prototype</i>. <p>
  * 
  * Though not required, it is prudent to supply a <i>copy constructor</i>
  * for any type implementing the {@code Copyable} interface. <p>
  *
  * <b>Implementation notes</b>:<br>
  * Unlike {@link Copyable}, this interface will return an instance of the
  * copyable type itself.  
  * 
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  *
  * @param  <T> The type of copyable object. 
  */
@Participant("Prototype")
public interface StrictCopyable<T extends StrictCopyable<T>> extends Copyable<T> {
    
    // Functionality inherited
}
