/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.visitor;

import dk.rode.thesis.meta.Participant;
import dk.rode.thesis.meta.model.Sequence;

/**
  * A <i>type visitable sequence</i> allows visitation 
  * by {@link SequenceTypeVisitor type} visitors. <p>
  * 
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  * 
  * @see    SequenceTypeScanner
  * 
  * @param  <E> The type of values delivered by this sequence. 
  */
@Participant("Element")
public interface TypeVisitableSequence<E> extends Sequence<E> {

    /**
      * Visits this visitable sequence using the 
      * {@link SequenceTypeVisitor type} visitor supplied 
      * as {@code visitor}.
      * 
      * @param  visitor  The visitor; never null.
      * @param  argument A context supplied argument, if any.
      * @see    SequenceTypeScanner#scan(TypeVisitableSequence, SequenceTypeVisitor, Object)
      * 
      * @param <P> The type of visitor-supplied argument. Use {@code Void} 
      *            for visitors that do not need an additional argument.
      */
    public <P> void accept(SequenceTypeVisitor<P> visitor, P argument);
    
}
