/*
 * Created: May-August, 2007
 *
 * Evaluating Software Design Patterns
 * - the "Gang of Four" patterns implemented in Java 6
 *
 * Gunni Rode - http://www.rode.dk 
 *
 * Master's Thesis, Computer Science
 * Department of Computer Science,
 * The Faculty of Science,
 * University of Copenhagen
 * 
 * Feel free to use and/or modify the Java 6 source code 
 * developed for this thesis AT YOUR OWN RISK, but note 
 * that the source code comes WITHOUT ANY - and I do mean 
 * WITHOUT ANY - form of warranty WHAT SO EVER!
 *
 * The original thesis and source code are available at 
 * http://www.rode.dk/thesis
 *
 */
package dk.rode.thesis.visitor;

import dk.rode.thesis.facade.RandomSequence;
import dk.rode.thesis.meta.model.Sequence;

/**
  * A <i>visitable random sequence</i> is a {@link RandomSequence} that
  * is {@link ValueVisitableSequence value} visitable <b>only</b> using
  * the {@link SequenceValueVisitor#visitIntegerValued(Sequence, Object)}
  * visitation method.
  *
  * @author Gunni Rode / <a href="http://www.rode.dk">rode.dk</a>
  */
public class VisitableRandomSequence 
     extends RandomSequence 
  implements ValueVisitableSequence<Integer> {
    
    /**
      * Constructor.
      * 
      * @param  maximum The maximum random value. Must be larger than 1.
      * @throws IllegalArgumentException If {@code max} is smaller than 1.
      */
    public VisitableRandomSequence(int maximum) {
        super(maximum);
    }
    
    /**
      * Copy constructor.
      * 
      * @param  sequence The sequence to copy; cannot be null.
      * @throws NullPointerException If {@code sequence} is null.
      */
    public VisitableRandomSequence(RandomSequence sequence) {
        super(sequence);
    }

    /**
      *
      */
    public VisitableRandomSequence copy() {
        return new VisitableRandomSequence(this);
    }
    
    /**
      * @see SequenceValueVisitor#visitIntegerValued(Sequence, Object) 
      */
    public <P> void accept(SequenceValueVisitor<P> visitor, P argument) {
        visitor.visitIntegerValued(this, argument);
    }
    
}
